"use strict";
var P = require("bluebird");
var websocket_1 = require("websocket");
var system_logger_1 = require("../../logging/system-logger");
var protocol_handler_1 = require("../protocol/protocol-handler");
var resolve_service_request_1 = require("../requests/resolve-service-request");
var response_handler_1 = require("../protocol/response-handler");
var stream_service_proxy_1 = require("./stream-service-proxy");
var StreamPlatformClient = (function () {
    function StreamPlatformClient(host, port, options) {
        var _this = this;
        this.host = host;
        this.port = port;
        this.options = options || { secure: false, version: 1 };
        this.protocolHandler = new protocol_handler_1.ProtocolHandler(this.options.version || 1);
        this.responseHandlers = {};
        this.conn = new P(function (resolve, reject) {
            _this.connResolver = resolve;
            _this.connRejecter = reject;
        });
        this.conn.catch(function (err) {
            console.log("Unable to connect to the platform. Stream will not be available", err);
        });
    }
    StreamPlatformClient.prototype.initialize = function () {
        var _this = this;
        var retries = 0;
        var max_retries = process.env.NODE_ENV === 'production' ? 5 : 1;
        this.logger = system_logger_1.SystemLogger.get();
        // Create the web socket
        this.socket = new websocket_1.client();
        this.socket.on('connectFailed', function (error) {
            console.log("Platform is not available yet. Retrying in 2 seconds");
            retries++;
            if (retries <= max_retries) {
                setTimeout(function () {
                    // Establish connection to the platform
                    _this.socket.connect('ws://' + _this.host + ":" + _this.port, "v1");
                }, 2000);
            }
            else {
                _this.connRejecter(error);
            }
        });
        this.socket.on('connect', function (conn) {
            console.log("Connected to stream");
            conn.on('error', _this.handleConnectionError.bind(_this));
            conn.on('close', function () {
                console.log('v1 Connection Closed');
            });
            _this.connResolver(conn);
        });
        // Establish connection to the platform
        this.socket.connect('ws://' + this.host + ":" + this.port, "v1");
    };
    StreamPlatformClient.prototype.handleConnectionError = function (error) {
        this.logger.debug("Connection error", error);
    };
    StreamPlatformClient.prototype.connection = function () {
        return this.conn;
    };
    StreamPlatformClient.prototype.registerResponseHandler = function (conn, request) {
        // Establish a message listener that will be removed after the response has been received.
        this.responseHandlers[request.id] = new response_handler_1.ResponseHandler(conn, this.protocolHandler);
        return this.responseHandlers[request.id].waitForResponse(request);
    };
    StreamPlatformClient.prototype.removeResponseHandler = function (request) {
        delete this.responseHandlers[request.id];
    };
    StreamPlatformClient.prototype.resolveService = function (query) {
        var _this = this;
        return this.conn.then(function (connection) {
            var request = new resolve_service_request_1.ResolveServiceRequest(query);
            // Send the request to the stream platform gateway
            connection.sendBytes(_this.protocolHandler.encode(request));
            // Register our response handler
            return _this.registerResponseHandler(connection, request).then(function (response) {
                console.log("Received response", response);
                // Remove this response handler
                _this.removeResponseHandler(request);
                // Create our service proxy
                return new stream_service_proxy_1.StreamServiceProxy(_this, connection, response.data);
            });
        });
    };
    StreamPlatformClient.prototype.send = function (request) {
        var _this = this;
        this.conn.then(function (connection) {
            // Send the request to the stream platform gateway
            connection.sendBytes(_this.protocolHandler.encode(request));
        });
    };
    StreamPlatformClient.prototype.exec = function (request) {
        var _this = this;
        return this.conn.then(function (connection) {
            // Send the request to the stream platform gateway
            connection.sendBytes(_this.protocolHandler.encode(request));
            // Register our response handler
            return _this.registerResponseHandler(connection, request).then(function (response) {
                // Remove this response handler
                _this.removeResponseHandler(request);
                return response;
            });
        });
    };
    StreamPlatformClient.prototype.broadcast = function (request) {
        // Make sure we don't target a specific service
        request.service = null;
        return this.exec(request);
    };
    StreamPlatformClient.prototype.head = function (request) {
        request.options.simulate = true;
        return this.exec(request);
    };
    return StreamPlatformClient;
}());
exports.StreamPlatformClient = StreamPlatformClient;

//# sourceMappingURL=stream-platform-client.js.map
